{$MODE OBJFPC}
Unit USBUtil;

Interface
Uses LibUSB,SysUtils,Baseunix,Utils;

Type EUSBError    = class(Exception);
     EUSBTimeout  = class(EUSBError);

  { TUSBDevice }

  TUSBDevice = class
  protected
    FDev          : PUSBDevice;
    FConfig       : Longint;
    FInterface    : Longint;
    FAltInterface : Longint;
    FRetries      : LongInt;
    FHandle       : PUSBDevHandle;
    Procedure   Open;
    Procedure   Close;
    Function    Send(EP:LongInt;Const Buf;Length,Timeout:LongInt):LongInt;
    Function    Send(EP:Longint;St:String;Timeout:LongInt):LongInt;
    Function    Recv(EP:Longint;Out Buf;Length:LongInt;Timeout:LongInt) : LongInt;
  public
    Constructor Create(idVendor,idProduct:LongInt;AConfig,AInterface,AAltInterface:LongInt;ARetries:Longint=0); overload;
    Constructor Create(ADev:PUSBDevice;AConfig,AInterface,AAltInterface:LongInt;ARetries:Longint=0); overload;
    Destructor  Destroy; override;
  End;

  { TUSBDeviceWithoutFirmware }

  TUSBDeviceWithoutFirmware = class(TUSBDevice)
  protected
    Procedure Download(AFilename:String); virtual; abstract;
  public
    Constructor Create(idVendor,idProduct:LongInt;ARetries:Longint=0); virtual; abstract; overload;
  End;
  TUSBDeviceWithoutFirmwareClass = Class of TUSBDeviceWithoutFirmware;

  { TUSBDeviceWithFirmware }

  TUSBDeviceWithFirmware = class(TUSBDevice)
  protected
  public
    Constructor Create(idVendor,idProduct,idVendorUnconf,idProductUnconf:LongInt;
                       ADeviceWithoutFirmwareClass:TUSBDeviceWithoutFirmwareClass;
                       AFirmwareFilename:String;
                       AConfig,AInterface,AAltInterface:LongInt;ARetries:Longint=0); overload;
  End;



(*
{$IFDEF FPC}
{$IF FPC_VERSION = 1 AND FPC_RELEASE = 1}
{$ELSE}
{$IF FPC_VERSION = 2}
{$ELSE}
Const ESysETIMEDOUT = Sys_ETIMEDOUT;
      ESysEAGAIN    = Sys_EAGAIN;
{$ENDIF}
{$ENDIF}
{$ENDIF}
*)
Procedure Initialize; { init libusb }

Function  USBFindDevices(idVendor,idProduct:LongInt;Var Devs : PUSBDevice):Longint;
Function  USBFindDevicesWithRetries(USBVend,USBProd,Retries:LongInt) : PUSBDevice;
Procedure DisposeUSBDevices(Var Devs : PUSBDevice);
Function  USBOpenAndSetup(Dev:PUSBDevice;Config,TheInterface,AltInterface:LongInt) : PUSBDevHandle;
Function  USBOpenAndSetup(Dev:PUSBDevice;Config,TheInterface,AltInterface,Retries:LongInt) : PUSBDevHandle;

Function USBGetDeviceDescription(Dev : PUSBDevice; Handle:PUSBDevHandle) : AnsiString;

Function  USB_Send(Hnd:PUSBDevHandle;EP:LongInt;Const Buf;Length,Timeout:LongInt):LongInt;
Function  USB_Send(Hnd:PUSBDevHandle;EP:LongInt;St:String;Timeout:LongInt):LongInt;
Function  USB_Recv(Hnd:PUSBDevHandle;EP:LongInt;Out Buf;Length:LongInt;Timeout:LongInt) : LongInt;

Var (* errno : LongInt; cvar; external; { from libc } *)
    usb_error_errno : LongInt; cvar; external; { from libusb }
    usb_error_type  : LongInt; cvar; external; { from libusb }
    usb_error_str   : PChar;   cvar; external; { from libusb }
    

Implementation

Const Initialized : Boolean = false;

Procedure Initialize; { init libusb }
Begin
  if Initialized then Exit;
  usb_init; { no problems will occur }
  Initialized := true;
End;

{ returns count of found devices, -1 on error }
Function USBFindDevices(idVendor,idProduct:LongInt;Var Devs : PUSBDevice):Longint;
Var Busses : PUSBBus;
    Dev    : PUSBDevice;
    Found  : PUSBDevice;
Begin
  Result := -1; { signal an error }
  if usb_find_busses  < 0 then Exit;
  if usb_find_devices < 0 then Exit;
  Busses := usb_get_busses;

  Result := 0;
  While Busses <> Nil do
    Begin
      Dev := Busses^.Devices;
      While Dev <> Nil do
        Begin
          if (Dev^.Descriptor.idVendor  = idVendor ) and
             (Dev^.Descriptor.idProduct = idProduct) then
            Begin
              New(Found);
              Found^ := Dev^;
              Found^.Next := Devs;
              Found^.Prev := Nil;
              if Devs <> Nil then
                Devs^.Prev  := Found;
              Devs        := Found;
              Inc(Result);
            End;
          Dev := Dev^.Next;
        End;
      Busses := Busses^.Next;
    End;
End;

{ This function searches the USB bus for devices with adjustable retries and }
{ returns a linked list if them (PUSBDevice).                                }
Function USBFindDevicesWithRetries(USBVend,USBProd,Retries:LongInt) : PUSBDevice;
Var L : LongInt;
Begin
  Result := Nil;
  While Retries >= 0 do
    Begin
      L := USBFindDevices(USBVend,USBProd,Result);
      if (L = 0) and (Retries > 0) then fpSelect(0,Nil,Nil,Nil,1000)
      else break;
      Dec(Retries);
    End;
End;

Procedure DisposeUSBDevices(Var Devs : PUSBDevice);
Var D : PUSBDevice;
Begin
  While Devs <> Nil do
    Begin
      D := Devs;
      Devs := D^.Next;
      Dispose(D);
    End;
End;

Function USBGetStringDescriptor(Handle : PUSBDevHandle;Index:Byte;Lang : LongInt) : WideString;
Var Buf : PUSBStringDescriptor;
    Len : LongInt;
Const StringBufSize = 1+1+256*2;
Begin
  if Handle = Nil then
    Begin
      Result := '*ERROR*';
      Exit;
    End;
{WriteLn('Sending GetDescriptor, Handle = ',IntToHex(LongInt(Handle),8),', Index = ',Index,', Lang = ',Lang); }
  Buf := GetMem(StringBufSize);
  Len := usb_control_msg(
    Handle,
    USB_ENDPOINT_IN or USB_TYPE_STANDARD or USB_RECIP_DEVICE { bmRequestType },
    USB_REQ_GET_DESCRIPTOR {bRequest},
    (Index and $FF) or (USB_DT_STRING shl 8) { wValue },
    Lang { wIndex },
    @(Buf^),
    StringBufSize,
    300);
  if Len <> Buf^.bLength then
    WriteLn('  Error: Len = ',Len,' should be equal to bLength = ',Buf^.bLength);
  { calculate length of string }
  Len := (Len - 2) div 2;
  { copy string to result }
  SetLength(Result,Len);
  Move(Buf^.wData[0],Result[1],Len*2);
  FreeMem(Buf);
{  WriteLn('GetString: Len = ',Len,',  Result = ',Result,' (',Length(Result),')  usb_strerror = ',usb_strerror); }
End;

Function USBGetDeviceDescription(Dev : PUSBDevice; Handle:PUSBDevHandle) : AnsiString;
Var I,J    : LongInt;
    Lang   : LongInt;
Begin
  if Dev = Nil then
    Exit('');
  try
{WriteLn('Opening Device '+StrPas(Dev^.Filename)); }
//    Handle := USBOpenAndSetup(Dev,1,0,1,5);
{WriteLn('Done opening.'); }
  except
    on E : Exception do
      Begin
        WriteLn('Couldn''t open the USB Device '+StrPas(Dev^.Filename)+' for USBGetDeviceDescription: '+E.Message);
        Handle := Nil;
      End;
  End;
  { we use the first defined language as default }
  Lang := LongInt(USBGetStringDescriptor(Handle,0,0)[1]);
{  WriteLn(USBGetStringDescriptor(Handle,1,Lang));}

  Result := 'USB Device ' + PChar(@(Dev^.bus^.dirname)) + '/' + PChar(@(Dev^.filename));
  Result := Result + ^J + '  bLength            = ' + IntToStr(Dev^.Descriptor.bLength);
  Result := Result + ^J + '  bDescriptorType    = ' + IntToStr(Dev^.Descriptor.bDescriptorType);
  Result := Result + ^J + '  bcdUSB             = ' + IntToStr(Dev^.Descriptor.bcdUSB);
  Result := Result + ^J + '  bDeviceClass       = ' + IntToStr(Dev^.Descriptor.bDeviceClass);
  Result := Result + ^J + '  bDeviceSubClass    = ' + IntToStr(Dev^.Descriptor.bDeviceSubClass);
  Result := Result + ^J + '  bDeviceProtocol    = ' + IntToStr(Dev^.Descriptor.bDeviceProtocol);
  Result := Result + ^J + '  bMaxPacketSize0    = ' + IntToStr(Dev^.Descriptor.bMaxPacketSize0);
  Result := Result + ^J + '  idVendor           = $'+ IntToHex(Dev^.Descriptor.idVendor,4);
  Result := Result + ^J + '  idProduct          = $'+ IntToHex(Dev^.Descriptor.idProduct,4);
  Result := Result + ^J + '  bcdDevice          = ' + IntToHex(Dev^.Descriptor.bcdDevice,4);
  Result := Result + ^J + '  iManufacturer      = ' + IntToStr(Dev^.Descriptor.iManufacturer) + ' "' + USBGetStringDescriptor(Handle,Dev^.Descriptor.iManufacturer,Lang) + '"';
  Result := Result + ^J + '  iProduct           = ' + IntToStr(Dev^.Descriptor.iProduct)      + ' "' + USBGetStringDescriptor(Handle,Dev^.Descriptor.iProduct,     Lang) + '"';
  Result := Result + ^J + '  iSerialNumber      = ' + IntToStr(Dev^.Descriptor.iSerialNumber) + ' "' + USBGetStringDescriptor(Handle,Dev^.Descriptor.iSerialNumber,Lang) + '"';
  Result := Result + ^J + '  bNumConfigurations = ' + IntToStr(Dev^.Descriptor.bNumConfigurations);
  if Dev^.Config = Nil then Exit;
  Result := Result + ^J + 'Config:';
  Result := Result + ^J + '  bLength             = ' + IntToStr(Dev^.Config^.bLength);
  Result := Result + ^J + '  bDescriptorType     = ' + IntToStr(Dev^.Config^.bDescriptorType);
  Result := Result + ^J + '  wTotalLength        = ' + IntToStr(Dev^.Config^.wTotalLength);
  Result := Result + ^J + '  bNumInterfaces      = ' + IntToStr(Dev^.Config^.bNumInterfaces);
  Result := Result + ^J + '  bConfigurationValue = ' + IntToStr(Dev^.Config^.bConfigurationValue);
  Result := Result + ^J + '  iConfiguration      = ' + IntToStr(Dev^.Config^.iConfiguration) + ' "' + USBGetStringDescriptor(Handle,Dev^.Config^.iConfiguration,Lang) + '"';
  Result := Result + ^J + '  bmAttributes        = ' + IntToStr(Dev^.Config^.bmAttributes);
  Result := Result + ^J + '  MaxPower            = ' + IntToStr(Dev^.Config^.MaxPower);
  Result := Result + ^J + '  Interface           = $'+ IntToHex(PtrUInt(Dev^.Config^.TheInterface),8);
  Result := Result + ^J + '  extra               = $'+ IntToHex(Cardinal(Dev^.Config^.extra),8);
  Result := Result + ^J + '  extralen            = ' + IntToStr(Dev^.Config^.extralen);
  if Dev^.Config^.TheInterface = Nil then Exit;
  For I := 0 to Dev^.Config^.bNumInterfaces-1 do
    Begin
      Result := Result + ^J + '  Interface '+IntToStr(I)+':';
      Result := Result + ^J + '    num_altsetting = ' + IntToStr(Dev^.Config^.TheInterface^[I].num_altsetting);
      Result := Result + ^J + '    altsetting     = $'+ IntToHex(Cardinal(Dev^.Config^.TheInterface^[I].altsetting),8);
      For J := 0 to Dev^.Config^.TheInterface^[I].num_altsetting-1 do
        Begin
          if Dev^.Config^.TheInterface^[i].AltSetting = Nil then Continue;
          Result := Result + ^J + '    AltSetting '+IntToStr(J)+':';
          Result := Result + ^J + '      bLength            = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bLength);
          Result := Result + ^J + '      bDescriptorType    = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bDescriptorType);
          Result := Result + ^J + '      bInterfaceNumber   = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bInterfaceNumber);
          Result := Result + ^J + '      bAlternateSetting  = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bAlternateSetting);
          Result := Result + ^J + '      bNumEndpoints      = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bNumEndpoints);
          Result := Result + ^J + '      bInterfaceClass    = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bInterfaceClass);
          Result := Result + ^J + '      bInterfaceSubClass = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bInterfaceSubClass);
          Result := Result + ^J + '      bInterfaceProtocol = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].bInterfaceProtocol);
          Result := Result + ^J + '      iInterface         = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].iInterface) + ' "' + USBGetStringDescriptor(Handle,Dev^.Config^.TheInterface^[I].AltSetting^[J].iInterface,Lang) + '"';
          Result := Result + ^J + '      endpoint           = $'+ IntToHex(Cardinal(Dev^.Config^.TheInterface^[I].AltSetting^[J].endpoint),8);
          Result := Result + ^J + '      extra              = $'+ IntToHex(Cardinal(Dev^.Config^.TheInterface^[I].AltSetting^[J].extra),8);
          Result := Result + ^J + '      extralen           = ' + IntToStr(Dev^.Config^.TheInterface^[I].AltSetting^[J].extralen);
          { TODO: show endpoints }
        End;
    End;
 // usb_close(Handle);

{  Result := Result^J + '  ' + Dev^.Config^. }
End;

Type TUSBExceptionType = Class of Exception;

Function USBException(Proc:String;Ret:LongInt) : Exception;
Var Ex : TUSBExceptionType;
Begin
  Case -Ret of
    ESysETIMEDOUT : Ex := EUSBTimeout;
  else
    Ex := EUSBError;
  End;
  Result := Ex.CreateFmt('%s returns %d; %s',[Proc,Ret,usb_strerror]);
End;

Function USBOpenAndSetup(Dev:PUSBDevice;Config,TheInterface,AltInterface:LongInt) : PUSBDevHandle;
Var R : LongInt;
Begin
  usb_error_errno := 0;
  Result := usb_open(Dev);
  { WriteLn('Result = ',Cardinal(Result),'  ErrNo = ',usb_error_errno,' Config = ',Config,' Interface = ',TheInterface,' AltInterface = ',AltInterface); }
  if Result = Nil then raise USBException('usb_open',LongInt(Result));

  try
  { set configuration }
    usb_error_errno := 0;
    R := usb_set_configuration(Result,Config);
    { WriteLn('R = ',R,'  ErrNo = ',usb_error_errno); }
    if R < 0 then raise USBException('usb_set_configuration',R);

    { set interface }
    usb_error_errno := 0;
    R := usb_claim_interface(Result,TheInterface);
    { WriteLn('R = ',R,'  ErrNo = ',usb_error_errno); }
    if R < 0 then raise USBException('usb_claim_interface',R);

    { set alternate interface }
    usb_error_errno := 0;
    R := usb_set_altinterface(Result,AltInterface);
    { WriteLn('R = ',R,'  ErrNo = ',usb_error_errno); }
    if R < 0 then raise USBException('usb_set_altinterface',R);
  except
    on E : Exception do
      Begin
        { close the USB handle }
        usb_close(Result);
        raise; { reraise the exception }
      End;  
  End;
End;

Function USBOpenAndSetup(Dev:PUSBDevice;Config,TheInterface,AltInterface,Retries:LongInt) : PUSBDevHandle;
Var St : ShortString;
Begin
  Result := Nil;
  While Retries >= 0 do
    Begin
      try
        Result := USBOpenAndSetup(Dev,Config,TheInterface,AltInterface);
        St := 'Result = Nil';  { see below after the try-except block }
        break;
      except
        on E : Exception do
          Begin
            St := E.Message;
            { WriteLn('USBOpenAndSetupRetries: Exception in USBOpenAndSetup: ',St,'; retrying ',Retries,' times.'); }
            if Retries > 0 then fpSelect(0,Nil,Nil,Nil,1000);
            Dec(Retries);
            if Retries <= 0 then
              raise E;
          End;
      End;
    End;
  if Result = Nil then
    raise Exception.Create(St);
End;

Function USB_Send(Hnd:PUSBDevHandle;EP:LongInt;Const Buf;Length,Timeout:LongInt):LongInt;
Var Loops  : Integer;
    Sent   : LongInt;
    ToSend : LongInt;
    Again  : Boolean;
    P      : Pointer;
Begin
{  WriteLn('USB_Send: EP = ',EP,' St = ',St,' Length = ',Length(St));}
  Sent := 0;
  ToSend := Length;
  Loops := 10;
  P     := @Buf;
  repeat
    usb_error_errno := 0;
    Result := usb_bulk_write(Hnd,EP,P,ToSend,Timeout); { returns number of bytes written on success or < 0 on error }
    Dec(Loops);
    Again := False;
    { check if an error occured }
    if Result < 0 then
      Begin
WriteLn('Problem with usb_bulk_write: Result = ',Result,', usb_error_errno = ',usb_error_errno,
                                                        ', usb_error_type = ',usb_error_type {,
                                                        ', usb_error_str = ',usb_error_str });
        if (Loops > 0) and (usb_error_errno = ESysEAGAIN) then
          Begin
            Again := True;
            fpSelect(0,Nil,Nil,Nil,100);
          End;
      End
    else
      Begin
        Inc(Sent,Result);
        Inc(P,Result);
        Dec(ToSend,Result);
        if Sent < Length then Again := True;
      End;
    { WriteLn('USB_Send: Hnd = ',Cardinal(Hnd),'  Loops = ',Loops,'  ErrNo = ',usb_error_errno,'  Result = ',Result,'  Length = ',Length,'  Timeout = ',Timeout,'  Sent = ',Sent,'  ToSend = ',ToSend,'  Again = ',Again); }
  until not Again;
  if Result < 0 then 
    raise USBException('usb_bulk_write',Result);
End;

Function USB_Send(Hnd:PUSBDevHandle;EP:LongInt;St:String;Timeout:LongInt):LongInt;
Begin
  Result := USB_Send(Hnd,EP,St[1],Length(St),Timeout);
End;

Function USB_Recv(Hnd:PUSBDevHandle;EP:LongInt;Out Buf;Length:LongInt;Timeout:LongInt) : LongInt;
{Var I : LongInt;
    B : Array[0..63] of Char absolute Buf;}
Begin
  usb_error_errno := 0;
  Result := usb_bulk_read(Hnd,$80 or EP,@Buf,Length,Timeout);
  //if Result > 0 then HexDump(Buf,Result);
  //if Result < 0 then raise USBException('usb_bulk_read',Result);
  //if Result < 0 then WriteLn(Format('%s returns %d; %s',['usb_bulk_read',Result,usb_strerror]));
End;



{ TUSBDevice }

Constructor TUSBDevice.Create(idVendor, idProduct: LongInt;AConfig,AInterface,AAltInterface:LongInt;ARetries:Longint);
Var Devs : PUSBDevice;
Begin
  { search the USB bus for the desired devices }
  Devs := USBFindDevicesWithRetries(idVendor,idProduct,ARetries);
  if Devs = Nil then
    raise EUSBError.Create('No device '+IntToHex(idVendor,4)+':'+IntToHex(idProduct,4)+' found.');
  { initialize }
  Create(Devs,AConfig,AInterface,AAltInterface,ARetries);
  { dispose Devs, because we create a copy of it }
  DisposeUSBDevices(Devs);
End;

Constructor TUSBDevice.Create(ADev: PUSBDevice;AConfig,AInterface,AAltInterface:LongInt;ARetries:Longint);
Begin
  inherited Create;
  New(FDev);
  FDev^         := ADev^;             { we store a copy of it }
  FConfig       := AConfig;
  FInterface    := AInterface;
  FAltInterface := AAltInterface;
  FRetries      := ARetries;
  { Open a connection to the USB device }
  Open;
End;

Destructor TUSBDevice.Destroy;
Begin
  Close;
  Dispose(FDev);
  inherited Destroy;
End;

Procedure TUSBDevice.Open;
Begin
  if FHandle <> Nil then Exit;
  FHandle := USBOpenAndSetup(FDev,FConfig,FInterface,FAltInterface,FRetries);
End;

Procedure TUSBDevice.Close;
Begin
  if FHandle = Nil then Exit;
  usb_close(FHandle);
  FHandle := Nil;
End;

Function TUSBDevice.Send(Ep: Longint; Const Buf; Length, Timeout: Longint): Longint;
Begin
  Result := USB_Send(FHandle,Ep,Buf,Length,Timeout);
End;

Function TUSBDevice.Send(EP:Longint;St: String; Timeout: Longint): Longint;
Begin
  Result := USB_Send(FHandle,Ep,St,Timeout);
End;

Function TUSBDevice.Recv(Ep:Longint; Out Buf; Length: Longint; Timeout: Longint): Longint;
Begin
  Result := USB_Recv(fHandle,Ep,Buf,Length,Timeout);
End;

{ TUSBDeviceWithFirmware }

Constructor TUSBDeviceWithFirmware.Create(idVendor, idProduct, idVendorUnconf, idProductUnconf: LongInt;
  ADeviceWithoutFirmwareClass: TUSBDeviceWithoutFirmwareClass;
  AFirmwareFilename: String;
  AConfig, AInterface, AAltInterface: LongInt;ARetries: Longint);
Var Devs : PUSBDevice;
    DeviceWithoutFirmware : TUSBDeviceWithoutFirmware;
Begin
  { search the USB bus for the desired devices }
  Devs := USBFindDevicesWithRetries(idVendor,idProduct,0);
  if Devs = Nil then
    Begin
      { no configured devices found, search for unconfigured }
      DeviceWithoutFirmware := ADeviceWithoutFirmwareClass.Create(idVendorUnconf,idProductUnconf,ARetries); { raises an exception if none are found }
      DeviceWithoutFirmware.Download(AFirmwareFilename);
      DeviceWithoutFirmware.Free;
      { the device should now have its firmware }
    End;
  { initialize }
  inherited Create(idVendor,idProduct,AConfig,AInterface,AAltInterface,ARetries);
  { dispose Devs, because we create a copy of it }
  DisposeUSBDevices(Devs);
End;

End.

