{$MODE OBJFPC}
Unit Utils;

Interface
Uses SysUtils;

Const HexChars = '0123456789ABCDEF';

Function HexToInt(St:ShortString):Int64;
Function Str2Int(St:ShortString):LongInt;
Function StrReplace(St:String;Src,Dst:String):String;
Procedure WriteDebug(St:AnsiString);
Function GetHeapSize : SizeInt;
Procedure HexDump(Var Buf;Length:SizeUInt);
Function RemoveFileExt(Filename,Ext:String) : String;

Implementation

Function HexToInt(St:ShortString):Int64;
Var I,J : LongInt;
Begin
  Result := 0;
  For I := 1 to Length(St) do
    Begin
      J := Pos(St[I],HexChars);
      if J = 0 then
        raise EConvertError.CreateFmt('Wrong hex char "%s" in hex number "%s" at position %d.',[St[I],St,I]);
      Result := Result shl 4 or (J-1);
    End;
End;

Function Str2Int(St:ShortString):LongInt;
Begin
  try
    Result := StrToInt(St);
  except
    WriteLn('Utils.Str2Int was called with "',St,'"! Returning 0.');
    Result := 0;
  End;
End;

Function StrReplace(St:String;Src,Dst:String):String;
Var L : LongInt;
Begin
  Result := '';
  repeat
    L := Pos(Src,St);
    if L = 0 then Exit(Result+St);
    Result := Result+Copy(St,1,L-1)+Dst;
    St := Copy(St,L+Length(Src),Length(St));
  Until False;
End;

Procedure WriteDebug(St:AnsiString);
Begin
  WriteLn('[',FormatDateTime('dd.mm.yyyy hh:nn:ss',Now),'] ',St);
End;

Function GetHeapSize : SizeInt;
Var Status : TFPCHeapStatus;
Begin
  Status := GetFPCHeapStatus;
  Result := Status.CurrHeapUsed;
End;

Procedure HexDump(Var Buf; Length: SizeUInt);
Var I : SizeUInt;
Begin
  Write('0000: ');
  For I := 0 to Length-1 do
    Begin
      Write(IntToHex((PByte(@Buf + I))^,2),' ');
      if (I and $000F = $000F) and (I <> Length-1) then
        Begin
          WriteLn;
          Write(IntToHex(I+1,4),': ');
        End;
    End;
  WriteLn;
end;

Function RemoveFileExt(Filename,Ext:String) : String;
Begin
  if RightStr(Filename,Length(Ext)) = Ext then
    Result := Copy(Filename,1,Length(Filename) - Length(Ext))
  else
    Result := Filename;
End;

End.

